package ueb042midisynth;

import processing.core.PApplet;

import javax.sound.midi.*;

@SuppressWarnings("serial")
public class Ueb042MidiSynth extends PApplet {

	private Synthesizer synthesizer;
	private Receiver receiver;

	private final int instrument = 3;

	private final int keyCount = 30;
	private final int keyStart = 50;
	private final int keyEnd = 100;

	private int key = -1;
	private int pitch = -1;
	private ShortMessage message;

	public void setup() {

		size(800, 300);

		initializeSynthesizer();
	}

	public void draw() {

		stroke(color(40, 40, 40));
		line(0, height - 80, width, height - 80);

		strokeWeight(3);

		int stepSize = width / keyCount;
		for (int index = 0; index <= keyCount; index++) {

			if (key == index) {
				fill(color(255, 200, 200));
			} else {
				fill(color(255, 255, 255));
			}

			line(stepSize * index, height - 80, stepSize * index, height);
			rect(stepSize * index, height - 80, stepSize, 80);
		}

	}

	public void mousePressed() {

		if (mouseY < height - 80)
			return;

		key = (int) (keyCount / (float) width * mouseX);

		play();
	}

	public void mouseReleased() {

		off();
	}

	public void keyPressed() {

		if (keyCode >= 65 && keyCode <= 84) {
			key = keyCode - 65;

			play();
		}
	}

	public void keyReleased() {

		off();
	}

	/**
	 * Spielt einen Ton ab
	 */
	private void play() {
		pitch = (int) (keyStart + (keyEnd - keyStart) / (float) (keyCount - 1) * key);

		System.out.println("Playing key " + key + " and pitch " + pitch);

		try {
			// Ereignis
			message = new ShortMessage();

			// Iteration
			// Note an
			message.setMessage(ShortMessage.NOTE_ON, 0, pitch, 100);
			receiver.send(message, -1);
		} catch (Exception e) {
			// Fehlerbehandlung
		}
	}

	/**
	 * Stoppt das Abspielen eines Tons
	 */
	private void off() {
		if (message != null && pitch != -1) {

			// Note aus
			try {
				message.setMessage(ShortMessage.NOTE_OFF, 0, pitch, 100);

				// senden
				receiver.send(message, -1);
			} catch (InvalidMidiDataException e) {
				e.printStackTrace();
			}
		}

		message = null;
		pitch = -1;
		key = -1;
	}

	/**
	 * Initialisiert den Synthesizer
	 */
	private void initializeSynthesizer() {

		// Den Synthesizer initialisieren
		try {

			synthesizer = MidiSystem.getSynthesizer();
			synthesizer.open();

			receiver = synthesizer.getReceiver();

			// Ereignis
			ShortMessage msg = new ShortMessage();

			// Instrument whlen
			msg.setMessage(ShortMessage.PROGRAM_CHANGE, 0, instrument, 31);
			receiver.send(msg, -1);

			// senden
			receiver.send(msg, -1);
		} catch (Exception ex) {
			ex.printStackTrace();

			exit();
			return;
		}
	}
}
